/*
 * Controller.cpp
 *
 *  Created on: Feb 3, 2015
 *      Author: rmin
 */

#include "Controller.h"
#include <chrono>

#define TIMEOUT_FOR_TRY_TO_CONNECT 1000

Controller::Controller(std::string settingsFile) :
		settingsFile(settingsFile) {
	// TODO Auto-generated constructor stub
	time = 0;
	port = 0;
	eventHandler = NULL;
	model = NULL;
	network = NULL;
	client = NULL;
}

Controller::~Controller() {
	if (eventHandler != NULL)
		delete eventHandler;
	if (model != NULL)
		delete model;
	if (network != NULL)
		delete network;
	if (client != NULL)
		delete client;
	// TODO Auto-generated destructor stub
}

void Controller::start() {
	try {
		readClientData();
		eventHandler = new EventHandler();
		model = new Model(eventHandler);
		client = new AI();
		network = new Network(this);
		network->setConnectionData(ip, port, token);
		int counter = 0;
		while (counter < 10 && network != NULL && !network->getIsConnected()) {
			counter++;
			network->connect();
			if (network->getIsTerminated() == true)
				break;
			boost::this_thread::sleep_for(
					boost::chrono::milliseconds(TIMEOUT_FOR_TRY_TO_CONNECT));
		}
		/*
		 * Connection Terminated
		 */

		delete eventHandler;
		delete model;
		delete client;
		delete network;
	} catch (const char* str) {
		std::cerr << str << std::endl;
	}
}

void Controller::readClientData() {
	std::ifstream fin(settingsFile.c_str());
	if (!fin.is_open()) {
		throw "Connection.conf doesn't exist";
	}
	std::string str;
	char tmp[1000];
	while (fin.getline(tmp, 1000)) {
		str.append(tmp);
	}
	Json::Value msg;
	Json::Reader reader;
	reader.parse(str,msg);
	token = msg["token"].asString();
	ip = msg["ip"].asString();
	port = msg["port"].asInt();
}

void Controller::handleMessage(Message &msg) {
	if (msg.getName() == "turn") {
		handleTurnMessage(msg);
	} else if (msg.getName() == "init")
		handleInitMessage(msg);
	else if (msg.getName() == "shutdown" || msg.getName() == "wrong token") {
		network->terminate();
	}
}

void Controller::handleTurnMessage(Message &msg) {
	model->handleTurnMessage(msg);
	doTurn();
}

void Controller::handleInitMessage(Message &msg) {
	model->handleInitMessage(msg);
}

void Controller::run() {
	client->doTurn(model->getWorld());
}

void Controller::doTurn() {
	try {

		boost::thread thr(boost::bind(&Controller::run, this));
		boost::this_thread::sleep_for(
				boost::chrono::milliseconds(MAX_TIME_FOR_DO_TURN));

		Message events = eventHandler->getEvents();
		eventHandler->clear();
		this->network->send(events);

	} catch (...) {
		std::cerr << "!!!\n";
	}
}
